/* global openWindow, Util, Configuration, ZhuAclMessages */
function ZhuAclController(ScreenService, templateManager, metaDataFactory, ZHPromise, ZhuAclService, ZhuAccessButtonService, ZhuAclConfig, MessageService, ApplicationContext, eventAggregator){
	var self = this;
	var user;
	var validProperties = ['widgets', 'fields', 'actions'];
	var loadedContainers = {};

	function User(userId, projectId, organizationId, defaultRule){
		var currentGroupId;
		var userDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;

		this.setCurrentGroupId = function(id){
			currentGroupId = id;
		};

		this.getDefaultRule = function(){
			return userDefaultRule;
		};

		this.getCurrentGroupId = function(){
			return currentGroupId;
		};

		this.getProjectId = function(){
			return projectId;
		};

		this.getUserId = function(){
			return userId;
		};

		this.getOrganizationId = function(){
			return organizationId;
		};
	}

	this.getDefaultRule = function(){
		return user.getDefaultRule();
	};

	this.getCurrentGroupId = function(){
		return user.getCurrentGroupId();
	};

	this.getProjectId = function(){
		return user.getProjectId();
	};

	var prepareLabelToName = function(label){
		var name = label.toLowerCase();
		name = name.replace(/\ /g, "_");
		return name.replace(/[^\w\s]/gi, '');
	};

	var translateLabel = function(label){
		return ScreenService.i18n(label);
	};

	this.getRulesFromDatasource = function(rulesData){
		var userId = user.getUserId();
		var rules = {};
		rulesData.forEach(function(row){
			var obj = {
				ruleRow: row,
				userGroups: {}
			};
			obj.userGroups[userId] = {
				rule: row.RULE
			};
			rules[row.ELEMENT_UNIQUE_ID] = obj;
		});
		return rules;
	};

	this.getElementRule = function(rules, itemId){
		var itemRule = {};
		if(rules[itemId]){
			itemRule = rules[itemId].ruleRow;
		}
		return itemRule;
	};

	this.getPlannedItemToInsertInDataSet = function(rules, item, itemType, parentId){
		var uniqueId = item.id;
		var itemRule = self.getElementRule(rules, uniqueId);
		var itemToInsert = {
			"UNIQUE_ID": uniqueId,
			"NAME": item.name || prepareLabelToName(item.label),
			"LABEL": translateLabel(item.label || item.name),
			"TYPE": itemType,
			"RULE": itemRule.RULE || this.getDefaultRule(),
			"RULE_ID": itemRule.ID,
			"PARENT_ID": parentId,
			"USER_ID": user.getUserId(),
			"PROJECT_ID": user.getProjectId(),
			"ORGANIZATION_ID": user.getOrganizationId()
		};

		itemToInsert.isVisible = item.aclIsVisible ? item.aclIsVisible : checkChildrenVisible(item, true);
		itemToInsert.isEditable = !item.aclReadOnly && item.aclIsVisible ? true : checkChildrenVisible(item);

		return itemToInsert;
	};

	var checkChildrenVisible = function(component, isVisibleHandle) {
		var control = false;
		var childrenTypes = ['widgets', 'fields', 'notifications'];
		if(!(component instanceof Container)){
			childrenTypes.push('actions');
		}
		childrenTypes.forEach(function(type){
			if(component[type]) {
				component[type].forEach(function(children){
					if (children && children.aclIsVisible && isVisibleHandle) {
						control = true;
					}
					if (children && children.aclIsVisible && !children.aclReadOnly) {
						control = true;
					}
				});
			}
		});
		return control;
	};

	var insertContainerElementsListWidget = function(container){
		return getFactoredWidget(container, ZhuAclConfig.containerElementsListWidgetName, true).then(function(factoredWidget){
			container.actions = container.actions || [];
			return factoredWidget;
		});
	};

	this.openAclModalWidget = function(){
		insertContainerElementsListWidget(templateManager.container).then(function(widget){
			ScreenService.openPopup(widget);
		});
	};

	var redirectToMainWindow = function(){
		var dashboardMenu = templateManager.project.getMenu('dashboard');
		var openLinkByMenu = ApplicationContext.LibUtilities && ApplicationContext.LibUtilities.openLinkByMenu;
		if(dashboardMenu && openLinkByMenu){
			openLinkByMenu(dashboardMenu);
		} else {
			openWindow(templateManager.project.mainWindow);
		}
	};

	var verifyContainerRule = function(rule){
		var containerIsRestrict = rule === ZhuAclConfig.RULE_INVISIBLE;
		if(containerIsRestrict){
			var actions = [{
				label: ScreenService.i18n("Ok"),
				code: redirectToMainWindow
			}];
			var dialogConfig = {
				type: 'ALERT',
				icon: 'exclamation',
				actions: actions
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.ACCESS_DENIED, {}, dialogConfig);
		}
		return !containerIsRestrict;
	};

	var mergeUndefinedRules = function(groupIds, itemRule, parentRules){
		parentRules = parentRules || {};

		var definedRule = itemRule.rule;
		var groupRules = parentRules.groupRules || {};

		if(!definedRule){
			definedRule = parentRules.rule || templateManager.project.aclDefaultRule;
		} else {
			var undefinedGroups = itemRule.undefinedGroups || [];
			groupIds.forEach(function(groupId){
				var parentRule = groupRules[groupId] || templateManager.project.aclDefaultRule;
				var isUndefinedGroup = undefinedGroups.indexOf(groupId) !== -1;
				var groupRule = isUndefinedGroup ? parentRule : definedRule;
				groupRules[groupId] = groupRule;

				if(definedRule !== ZhuAclConfig.RULE_ALLOW && groupRule !== ZhuAclConfig.RULE_INVISIBLE){
					definedRule = groupRule;
				}
			});
			itemRule.rule = definedRule;
		}

		return {rule: definedRule, groupRules: groupRules};
	};

	var getUserRule = function(rules, userId, elementId){
		return Util.getNestedProperty(rules, [elementId, 'userGroups', userId]);
	};

	var initObject = function(obj, objKeys, lastKeyValue){
		var objKeysLength = objKeys.length;
		objKeys.reduce(function(prev, next, index){
			if(index == objKeysLength - 1){
				prev[next] = lastKeyValue;
			} else {
				prev[next] = prev[next] || {};
			}
			return prev[next];
		}, obj);
	};

	var updateContainerAcl = function(rules, groupIds, userId){
		var applyRule = function(item, parentRules){
			var id = item.id;
			var userRule = getUserRule(rules, userId, id) || {};
			var mergedItem = mergeUndefinedRules(groupIds, userRule, parentRules);
			var itemRule = mergedItem.rule;

			item.aclIsVisible = itemRule != ZhuAclConfig.RULE_INVISIBLE;
			item.aclReadOnly = itemRule != ZhuAclConfig.RULE_ALLOW;

			return mergedItem;
		};

		var updateItemAcl = function(item, type, parentRules){
			var mergedItem = applyRule(item, parentRules);

			ZhuAclConfig.getAclMap(type).childs.forEach(function(type){
				if(Util.isArray(item[type])){
					item[type].forEach(function(child){
						updateItemAcl(child, type, mergedItem);
					});
				}
			});
		};

		var changeNotificationVisibility = function(isVisible){
			templateManager.project.notifications.some(function(notification){
				var isAclNotification = notification.name === "accessControl";
				if(isAclNotification){
					notification.isVisible = isVisible;
				}
				return isAclNotification;
			});
		};

		var updateUserRules = function(container){
			var containerId = container.id;
			var parentMenuId = container.parentMenuId;
			var userRule = getUserRule(rules, userId, containerId);
			if(!userRule && parentMenuId) {
				userRule = getUserRule(rules, userId, parentMenuId);
				if(userRule){
					initObject(rules, [containerId, 'userGroups', userId], userRule);
				}
			}
		};

		var container = templateManager.container;
		var aclEnable = container.aclEnable;
		aclEnable = aclEnable !== undefined ? aclEnable : true;
		changeNotificationVisibility(aclEnable);

		if(aclEnable){
			updateUserRules(container);
			var containerRule = applyRule(container);
			if(verifyContainerRule(containerRule.rule)){
				updateItemAcl(container, ZhuAclConfig.containerType);
			}
			loadedContainers[container.id] = true;
		}
	};

	var convertObjectRulesToArray = function(userId, objRules){
		var arrayRules = [];
		Object.keys(objRules).forEach(function(ruleId){
			var objRule = objRules[ruleId];
			var userRule = objRule.userGroups[userId];
			if(userRule){
				var rule = {
					"UNIQUE_ID": ruleId,
					"NAME": objRule.itemName,
					"RULE": userRule.rule
				};
				arrayRules.push(rule);
			}
		});
		return arrayRules;
	};

	var convertArrayRulesToObject = function(userId, arrayRules){
		var objRules = {};
		arrayRules.forEach(function(rule){
			var profileObjects = {};
			profileObjects[userId] = {
				'rule': rule.RULE,
				'undefinedGroups': rule.GROUP_ID || []
			};
			objRules[rule.UNIQUE_ID] = {
				'itemName': rule.NAME,
				'userGroups': profileObjects
			};
		});
		return objRules;
	};

	var subscribeContainerEvent = function(rules, groupIds, userId){
		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function applyACL(){
			var containerId = templateManager.container.id;
			if(!loadedContainers[containerId]){
				updateContainerAcl(rules, groupIds, userId);
			}
		});

	};

	var updateProjectAcl = function(loginRules, defaultRule, groupIds){
		var rules = {};
		var userId = user.getUserId();
		if(Util.isObject(loginRules)){
			rules = loginRules;
			loginRules = convertObjectRulesToArray(userId, rules);
		} else if(Util.isArray(loginRules)){
			rules = convertArrayRulesToObject(userId, loginRules);
		}

		templateManager.project.aclDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;
		templateManager.userId = userId;

		templateManager.aclRules = rules;
		updateContainerAcl(rules, groupIds, userId);
		subscribeContainerEvent(rules, groupIds, userId);
	};

	var createUser = function createUser(userId, projectId, organizationId, defaultRule){
		user = new User(userId, projectId, organizationId, defaultRule);
	};

	this.setUserRules = function(groupIds, projectId, userId, organizationId, options){
		var returnPromise;
		loadedContainers = {};
		options = options || {};

		if(options.canConfigureAcl){
			ZhuAccessButtonService.enableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		} else {
			ZhuAccessButtonService.disableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		}

		if(options.rules){
			createUser(userId, projectId, organizationId, options.defaultRule);
			updateProjectAcl(options.rules, options.defaultRule, groupIds);
			returnPromise = ZHPromise.when(templateManager.aclRules);
		} else {
			returnPromise = ZhuAclService.getZhuAclRules(projectId, groupIds).then(function(response){
				createUser(userId, projectId, organizationId, response.defaultRule);
				updateProjectAcl(response.aclRules, response.defaultRule, groupIds);
				return templateManager.aclRules;
			}).catch(function(err){
				MessageService.showMessageByCode(err.code);
			});
		}
		return returnPromise;
	};

	this.setElementsListFilter = function(container, groupId, widgetName){
		widgetName = widgetName || ZhuAclConfig.menuElementsListWidgetName;
		var elementsWidget = container.getWidget(widgetName);
		elementsWidget.dataSourceFilter = ZhuAclConfig.getDefaultDataSourceFilter(groupId, user.getProjectId(), user.getOrganizationId());
		user.setCurrentGroupId(groupId);
	};

	var getFactoredWidget = function(container, widgetName, pushInContainer){
		var widget = container.getWidget(widgetName);
		if(widget){
			return ZHPromise.when(widget);
		} else {
			return ZhuAclService.getWidgetMetadata(widgetName).then(function(metadata){
				var factoredWidget = metaDataFactory.widgetFactory(metadata, container);
				if(pushInContainer){
					container.widgets.push(factoredWidget);
				}
				return factoredWidget;
			});
		}
	};
	this.getFactoredWidget = getFactoredWidget;

	this.insertElementsListWidget = function(container, parentWidget, extraElementsPath){
		getFactoredWidget(container, ZhuAclConfig.menuElementsListWidgetName).then(function(factoredWidget) {
			if(parentWidget && !factoredWidget.parent){
				factoredWidget.parent = parentWidget;
				parentWidget.widgets = parentWidget.widgets || [];
				parentWidget.widgets.push(factoredWidget);
				factoredWidget.extraElementsPath = extraElementsPath;
			}
		});
	};

}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclController', ZhuAclController);
});
